@@ -1,6 +1,6 @@
-from numpy import zeros, ones, ndarray, count_nonzero
+from numpy import zeros, ones, ndarray
 from networkx import adjacency_matrix, Graph
-from scipy.sparse import diags, lil_matrix, spmatrix, csr_matrix
+from scipy.sparse import diags, lil_matrix, spmatrix
 from scipy.sparse.linalg import lsqr
 
 
@@ -38,7 +38,7 @@ def forward_hierarchical_levels(graph, weight=None):
         k_in = A.sum(axis=1).A1
        
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight).transpose()
+        A = adjacency_matrix(graph, weight=weight).transpose()
         k_in = A.sum(axis=1).A1
         
         
@@ -82,7 +82,7 @@ def backward_hierarchical_levels(graph, weight=None):
         k_in = A.sum(axis=1).A1
        
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight)
+        A = adjacency_matrix(graph, weight=weight)
         k_in = A.sum(axis=1).A1
         
         
@@ -116,7 +116,7 @@ def hierarchical_levels(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    return forward_hierarchical_levels(graph, weight) - backward_hierarchical_levels(graph, weight)
+    return forward_hierarchical_levels(graph, weight=weight) - backward_hierarchical_levels(graph, weight=weight)
 
 
 
@@ -128,9 +128,9 @@ def sparse_forward_hierarchical_differences(graph, weight=None):
         A = graph.transpose()
        
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight).transpose()
+        A = adjacency_matrix(graph, weight=weight).transpose()
     
-    s = forward_hierarchical_levels(graph, weight)
+    s = forward_hierarchical_levels(graph, weight=weight)
     TD = lil_matrix(A.shape, dtype=float)
     
     for i, j in zip(A.nonzero()[0], A.nonzero()[1]):
@@ -163,7 +163,7 @@ def forward_hierarchical_differences(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    TD = sparse_forward_hierarchical_differences(graph, weight)
+    TD = sparse_forward_hierarchical_differences(graph, weight=weight)
     return TD.toarray()
 
 
@@ -179,7 +179,7 @@ def sparse_backward_hierarchical_differences(graph, weight=None):
     elif isinstance(graph, Graph):
         A = adjacency_matrix(graph, weight=weight)
         
-    s = backward_hierarchical_levels(graph, weight)
+    s = backward_hierarchical_levels(graph, weight=weight)
     TD = lil_matrix(A.shape, dtype=float)
     
     for i, j in zip(A.nonzero()[0], A.nonzero()[1]):
@@ -212,8 +212,7 @@ def backward_hierarchical_differences(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    TD = sparse_backward_hierarchical_differences(graph, weight)
-   
+    TD = sparse_backward_hierarchical_differences(graph, weight=weight)
     return TD.toarray()
 
 
@@ -251,23 +250,17 @@ def forward_hierarchical_incoherence(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    if isinstance(graph, ndarray):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph.transpose()
-        tot = float(count_nonzero(A))
-        
-    elif isinstance(graph, spmatrix):
-        A = graph.transpose()
-        tot = float(A.count_nonzero())
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight).transpose()
-        tot = float(A.count_nonzero())
+        A = adjacency_matrix(graph, weight=weight).transpose()
         
     TD = sparse_forward_hierarchical_differences(graph, weight=weight).tocsr()
-    m = (TD.sum()) / tot
+    m = (TD.sum()) / A.sum()
     
     TD2 = TD.power(2)
-    m2 = (TD2.sum()) / tot
+    m2 = (TD2.sum()) / A.sum()
     
     std = (m2 - m**2)**0.5    
     return TD, m, std
@@ -306,23 +299,17 @@ def backward_hierarchical_incoherence(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    if isinstance(graph, ndarray):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph
-        tot = float(count_nonzero(A))
-        
-    elif isinstance(graph, spmatrix):
-        A = graph
-        tot = float(A.count_nonzero())
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight)
-        tot = float(A.count_nonzero())
+        A = adjacency_matrix(graph, weight=weight)
         
-    TD = sparse_backward_hierarchical_differences(graph, weight)
-    m = (TD.sum()) / tot
+    TD = sparse_backward_hierarchical_differences(graph, weight=weight)
+    m = (TD.sum()) / A.sum()
     
     TD2 = TD.power(2)
-    m2 = (TD2.sum()) / tot
+    m2 = (TD2.sum()) / A.sum()
     
     std = (m2 - m**2)**0.5    
     return TD, m, std
@@ -355,20 +342,14 @@ def forward_democracy_coefficient(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    if isinstance(graph, ndarray):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph.transpose()
-        tot = float(count_nonzero(A))
-        
-    elif isinstance(graph, spmatrix):
-        A = graph.transpose()
-        tot = float(A.count_nonzero())
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight).transpose()
-        tot = float(A.count_nonzero())
+        A = adjacency_matrix(graph, weight=weight).transpose()
         
-    TD = sparse_forward_hierarchical_differences(graph, weight)
-    m = (TD.sum()) / tot
+    TD = sparse_forward_hierarchical_differences(graph, weight=weight)
+    m = (TD.sum()) / A.sum()
     
     return 1 - m
 
@@ -399,20 +380,14 @@ def backward_democracy_coefficient(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    if isinstance(graph, ndarray):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph
-        tot = float(count_nonzero(A))
-        
-    elif isinstance(graph, spmatrix):
-        A = graph
-        tot = float(A.count_nonzero())
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight)
-        tot = float(A.count_nonzero())
+        A = adjacency_matrix(graph, weight=weight)
         
-    TD = sparse_backward_hierarchical_differences(graph, weight)
-    m = (TD.sum()) / tot
+    TD = sparse_backward_hierarchical_differences(graph, weight=weight)
+    m = (TD.sum()) / A.sum()
     
     return 1 - m
 
@@ -444,23 +419,17 @@ def node_forward_influence_centrality(graph, node, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    if isinstance(graph, ndarray):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph.transpose()
-        tot = float(count_nonzero(A[node]))
-        
-    elif isinstance(graph, spmatrix):
-        A = graph.transpose()
-        tot = float(A[node].count_nonzero())
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight).transpose()
-        tot = float(A[node].count_nonzero())
+        A = adjacency_matrix(graph, weight=weight).transpose()
         
-    TD = sparse_forward_hierarchical_differences(graph, weight).tocsr()
+    TD = sparse_forward_hierarchical_differences(graph, weight=weight).tocsr()
     if A[node].sum() == 0:
         m = 0
     else:
-        m = (TD[node].sum()) / tot
+        m = (TD[node].sum()) / A[node].sum()
     return 1 - m
 
 
@@ -493,23 +462,17 @@ def node_backward_influence_centrality(graph, node, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    if isinstance(graph, ndarray):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph
-        tot = float(count_nonzero(A[node]))
-        
-    elif isinstance(graph, spmatrix):
-        A = graph
-        tot = float(A[node].count_nonzero())
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight)
-        tot = float(A[node].count_nonzero())
+        A = adjacency_matrix(graph, weight=weight)
         
-    TD = sparse_backward_hierarchical_differences(graph, weight).tocsr()
+    TD = sparse_backward_hierarchical_differences(graph, weight=weight).tocsr()
     if A[node].sum() == 0:
         m = 0
     else:
-        m = (TD[node].sum()) / tot
+        m = (TD[node].sum()) / A[node].sum()
     return 1 - m
 
 
@@ -537,23 +500,20 @@ def forward_influence_centrality(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
     
-    if isinstance(graph, ndarray):
-        A = csr_matrix(graph.transpose())
-        
-    elif isinstance(graph, spmatrix):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph.transpose()
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight).transpose()
+        A = adjacency_matrix(graph, weight=weight).transpose()
         
-    TD = sparse_forward_hierarchical_differences(graph, weight).tocsr()
+    TD = sparse_forward_hierarchical_differences(graph, weight=weight).tocsr()
     m = zeros((TD.shape[0], 1))
     
     for i in range(m.shape[0]):
         if A[i].sum() == 0:
             m[i] = 0
         else:
-            m[i] = (TD[i].sum()) / float(A[i].count_nonzero())
+            m[i] = (TD[i].sum()) / A[i].sum()
     return ones((m.shape[0], 1)) - m
 
 
@@ -581,23 +541,20 @@ def backward_influence_centrality(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358."""
         
-    if isinstance(graph, ndarray):
-        A = csr_matrix(graph)
-        
-    elif isinstance(graph, spmatrix):
+    if isinstance(graph, (ndarray, spmatrix)):
         A = graph
         
     elif isinstance(graph, Graph):
-        A = adjacency_matrix(graph, weight)
+        A = adjacency_matrix(graph, weight=weight)
         
-    TD = sparse_forward_hierarchical_differences(graph, weight).tocsr()
+    TD = sparse_forward_hierarchical_differences(graph, weight=weight).tocsr()
     m = zeros((TD.shape[0], 1))
     
     for i in range(m.shape[0]):
         if A[i].sum() == 0:
             m[i] = 0
         else:
-            m[i] = (TD[i].sum()) / float(A[i].count_nonzero())
+            m[i] = (TD[i].sum()) / A[i].sum()
     return ones((m.shape[0], 1)) - m
 
 
@@ -641,9 +598,9 @@ def forward_hierarchical_metrics(graph, weight=None):
     Graph hierarchy and spread of infections. 
     arXiv preprint arXiv:1908.04358.
     '''
-    s = forward_hierarchical_levels(graph, weight)
-    ic = forward_influence_centrality(graph, weight)
-    a = forward_hierarchical_incoherence(graph, weight)
+    s = forward_hierarchical_levels(graph, weight=weight)
+    ic = forward_influence_centrality(graph, weight=weight)
+    a = forward_hierarchical_incoherence(graph, weight=weight)
     
     return s, ic, a[0], 1 - a[1], a[2] 
 
@@ -688,9 +645,9 @@ def backward_hierarchical_metrics(graph, weight=None):
     arXiv preprint arXiv:1908.04358.
     '''
     
-    s = backward_hierarchical_levels(graph, weight)
-    ic = backward_influence_centrality(graph, weight)
-    a = backward_hierarchical_incoherence(graph, weight)
+    s = backward_hierarchical_levels(graph, weight=weight)
+    ic = backward_influence_centrality(graph, weight=weight)
+    a = backward_hierarchical_incoherence(graph, weight=weight)
     
     return s, ic, a[0], 1 - a[1], a[2] 
 
